#include<avr/io.h>
#include <avr/interrupt.h>
#include "lcd.h"
#include "buck.h"


volatile unsigned char pwmF = 1,pwmD = 5;


void pwm_init(void)
{
	DDRB |= 0x60;	// OCR1A, OCR1B 
	// PWM mode 14
	// 8 , 14.7456MHz
	TCCR1A = 0xB2;	//PWM, Fast PWM
	TCCR1B = 0x1A;
	TCCR1C = 0x00;
}

void pwm_set(unsigned int pwmFreq,unsigned int pwmDuty)
{
	pwmFreq = 1843200/pwmFreq;
	pwmDuty = (unsigned long)pwmFreq*pwmDuty/100;
	// set top count value
	ICR1H = (unsigned char)(pwmFreq>>8);
	ICR1L = (unsigned char)pwmFreq;

	OCR1AH = (unsigned char)(pwmDuty>>8);
	OCR1AL = (unsigned char)pwmDuty;

	OCR1BH = (unsigned char)(pwmDuty>>8);
	OCR1BL = (unsigned char)pwmDuty;
}

void SW_init(void)
{
	DDRD = 0x00;
	EICRA = 0xFF;
	EICRB = 0x00;
	EIMSK = 0x0F;
	EIFR = 0x0F;
}

void adc_init(void)
{
	ADMUX=0x40;     
	ADCSR=0xA6;         
	ADCSR |= 0x40; //ADC start conversion
}

unsigned long adc_read(void)
{
	unsigned int adcValue=0;
	float adc_cv;

	while( (ADCSR&0x10)==0x00); //ADC ȯ   
	adcValue = ADCL | (ADCH << 8);

	adc_cv = (float)adcValue * 3.3 / 1023;//(113K+10.2K)/10.2K
	adc_cv = adc_cv * 12078;//(113K+10.2K)/10.2K -> 1232/102 , x 1000(Ҽ 3ڸ )
	return adc_cv;
}

void printf_3dot3(unsigned long data)
{
	if(data/10 == 0){
		lcdDataWrite('0' + (data%10));
		lcdDataWrite('m');
		lcdDataWrite(' ');
		lcdDataWrite(' ');
		lcdDataWrite(' ');
	}else if(data/100 == 0){
		lcdDataWrite('0' + (data/10));
		lcdDataWrite('0' + (data%10));
		lcdDataWrite('m');
		lcdDataWrite(' ');
		lcdDataWrite(' ');
	}else if(data/1000 == 0){
		lcdDataWrite('0' + (data/100));
		lcdDataWrite('0' + ((data/10)%10));
		lcdDataWrite('0' + (data%10));
		lcdDataWrite('m');
		lcdDataWrite(' ');
	}else if(data/10000 == 0)	{
		lcdDataWrite('0' + (data/1000));
		lcdDataWrite('.');
		lcdDataWrite('0' + ((data/100)%10));
		lcdDataWrite('0' + ((data/10)%10));
		lcdDataWrite('0' + (data%10));
	}else if(data/100000 == 0){
		lcdDataWrite('0' + (data/10000));
		lcdDataWrite('0' + ((data/100)%10));
		lcdDataWrite('.');
		lcdDataWrite('0' + ((data/100)%10));
		lcdDataWrite('0' + (data%10));
	}else{
		lcdDataWrite('0' + ((data/10)%10));
		lcdDataWrite('0' + (data%10));
	}
}

SIGNAL(SIG_INTERRUPT0)		// PWM Frequency Up
{
	if(pwmF < 30) pwmF++;
}

SIGNAL(SIG_INTERRUPT1)		// PWM Frequency Down
{
	if(1 < pwmF) pwmF--;
}

SIGNAL(SIG_INTERRUPT2)		// PWM Duty Up
{
	if(pwmD < 9) pwmD++;
}

SIGNAL(SIG_INTERRUPT3)		// PWM Duty Down
{
	if(1 < pwmD) pwmD--;
}
