#include<avr/io.h>
#include <avr/interrupt.h>
#include "lcd.h"
#include "boost.h"


volatile unsigned char pwmF = 10,pwmD = 5;
volatile unsigned char timer2overflow = 0, ADC_channel = 0;
volatile unsigned long adc_sum = 0;
volatile unsigned int adc_count = 0,adc_data[3];


void pwm_init(void)
{
	DDRB |= 0x20;	// OCR1A 
	// PWM mode 14
	// 14.7456MHz
	TCCR1A = 0x82;	//PWM, Fast PWM
	TCCR1B = 0x19;
	TCCR1C = 0x00;
}

void pwm_set(unsigned int pwmFreq,unsigned int pwmDuty)
{
	pwmFreq = F_CPU/pwmFreq;
	pwmDuty = (unsigned long)pwmFreq*pwmDuty/100;
	// set top count value
	ICR1H = (unsigned char)(pwmFreq>>8);
	ICR1L = (unsigned char)pwmFreq;

	OCR1AH = (unsigned char)(pwmDuty>>8);
	OCR1AL = (unsigned char)pwmDuty;

	OCR1AH = (unsigned char)(pwmDuty>>8);
	OCR1AL = (unsigned char)pwmDuty;
}

void SW_init(void)
{
	DDRD = 0x00;
	EICRA = 0xFF;
	EICRB = 0x00;
	EIMSK = 0x0F;
	EIFR = 0x0F;
}

void adc_init(void)
{
	ADMUX = 0x40;	// AVCC pin, internal VREF turned off
	ADCSRA = 0xAC;	// enable ADC, single convert mode, 1/16 prescaler
	TCCR2 = 0x03;	// Timer 2, 1/64 prescaler
	TIMSK |= 0x40;	// enable Timer 2 overflow interrupt
	TIFR |= 0x40;	// clear Timer 2 overflow interrupt flag
	ADCSRA |= 0x10;	// clear hardware "conversion complete" flag
	ADCSRA |= 0x40;	// ADC start conversion
}

unsigned long adc_read_Vin(void)
{
	float adc_cv;

	adc_cv = (float)adc_data[0] * 3.3 / 1023;
	adc_cv = adc_cv * 12078;//(113K+10.2K)/10.2K -> 123200/10200 , x 1000(Ҽ 3ڸ )
	return adc_cv;
}

unsigned long adc_read_Vout(void)
{
	float adc_cv;

	adc_cv = (float)adc_data[1] * 3.3 / 1023;
	adc_cv = adc_cv * 12078;//(113K+10.2K)/10.2K -> 123200/10200 , x 1000(Ҽ 3ڸ )
	return adc_cv;
}

void printf_3dot3(unsigned long data)
{
	if(data/10 == 0){
		lcdDataWrite('0' + (data%10));
		lcdDataWrite('m');
		lcdDataWrite(' ');
		lcdDataWrite(' ');
		lcdDataWrite(' ');
	}else if(data/100 == 0){
		lcdDataWrite('0' + (data/10));
		lcdDataWrite('0' + (data%10));
		lcdDataWrite('m');
		lcdDataWrite(' ');
		lcdDataWrite(' ');
	}else if(data/1000 == 0){
		lcdDataWrite('0' + (data/100));
		lcdDataWrite('0' + ((data/10)%10));
		lcdDataWrite('0' + (data%10));
		lcdDataWrite('m');
		lcdDataWrite(' ');
	}else if(data/10000 == 0)	{
		lcdDataWrite('0' + (data/1000));
		lcdDataWrite('.');
		lcdDataWrite('0' + ((data/100)%10));
		lcdDataWrite('0' + ((data/10)%10));
		lcdDataWrite('0' + (data%10));
	}else if(data/100000 == 0){
		lcdDataWrite('0' + (data/10000));
		lcdDataWrite('0' + ((data/100)%10));
		lcdDataWrite('.');
		lcdDataWrite('0' + ((data/100)%10));
		lcdDataWrite('0' + (data%10));
	}else{
		lcdDataWrite('0' + ((data/10)%10));
		lcdDataWrite('0' + (data%10));
	}
}

SIGNAL(SIG_INTERRUPT0)		// PWM Frequency Up
{
	if(pwmF < 30) pwmF++;
}

SIGNAL(SIG_INTERRUPT1)		// PWM Frequency Down
{
	if(1 < pwmF) pwmF--;
}

SIGNAL(SIG_INTERRUPT2)		// PWM Duty Up
{
	if(pwmD < 9) pwmD++;
}

SIGNAL(SIG_INTERRUPT3)		// PWM Duty Down
{
	if(1 < pwmD) pwmD--;
}

SIGNAL(SIG_OVERFLOW2)
{
	timer2overflow++;
	if(timer2overflow == 225)
	{
		adc_sum /= adc_count - 1;
		adc_data[ADC_channel] = adc_sum;
		adc_sum = 0;
		adc_count = 0;
		ADC_channel++;
		if(ADC_channel == 3) ADC_channel = 0;
		timer2overflow = 0;
	}
}

SIGNAL(SIG_ADC)
{
	if(adc_count == 0)
	{
		ADMUX = 0x40+ADC_channel;
	}
	else
	{
		adc_sum += ADC;
	}
	adc_count++;
}
